@@ -12,16 +12,15 @@ use crate::aspect_ratio::AspectRatio;
 use crate::bbox::BoundingBox;
 use crate::coord_units::CoordUnits;
 use crate::dasharray::Dasharray;
-use crate::document::{AcquiredNode, AcquiredNodes};
+use crate::document::AcquiredNodes;
 use crate::dpi::Dpi;
+use crate::element::Element;
 use crate::error::{AcquireError, RenderingError};
 use crate::filters;
-use crate::gradient::{LinearGradient, RadialGradient};
 use crate::marker;
-use crate::node::{CascadedValues, NodeDraw, NodeType, RsvgNode};
+use crate::node::{CascadedValues, Node, NodeBorrow, NodeDraw};
 use crate::paint_server::{PaintServer, PaintSource};
 use crate::path_builder::*;
-use crate::pattern::Pattern;
 use crate::properties::ComputedValues;
 use crate::property_defs::{
     ClipRule, FillRule, Opacity, Overflow, ShapeRendering, StrokeDasharray, StrokeLinecap,
@@ -29,7 +28,7 @@ use crate::property_defs::{
 };
 use crate::rect::Rect;
 use crate::shapes::Markers;
-use crate::structure::{ClipPath, Mask, Symbol, Use};
+use crate::structure::Mask;
 use crate::surface_utils::{
     shared_surface::ExclusiveImageSurface, shared_surface::SharedImageSurface,
     shared_surface::SurfaceType,
@@ -48,18 +47,16 @@ use crate::viewbox::ViewBox;
 /// returned `ViewParams` is dropped; at that point, the `DrawingCtx` will resume using its
 /// previous viewport.
 pub struct ViewParams {
-    pub dpi_x: f64,
-    pub dpi_y: f64,
+    pub dpi: Dpi,
     pub view_box_width: f64,
     pub view_box_height: f64,
     view_box_stack: Option<Weak<RefCell<Vec<ViewBox>>>>,
 }
 
 impl ViewParams {
-    pub fn new(dpi_x: f64, dpi_y: f64, view_box_width: f64, view_box_height: f64) -> ViewParams {
+    pub fn new(dpi: Dpi, view_box_width: f64, view_box_height: f64) -> ViewParams {
         ViewParams {
-            dpi_x,
-            dpi_y,
+            dpi,
             view_box_width,
             view_box_height,
             view_box_stack: None,
@@ -95,7 +92,7 @@ pub struct DrawingCtx {
 
     view_box_stack: Rc<RefCell<Vec<ViewBox>>>,
 
-    drawsub_stack: Vec<RsvgNode>,
+    drawsub_stack: Vec<Node>,
 
     measuring: bool,
     testing: bool,
@@ -103,7 +100,7 @@ pub struct DrawingCtx {
 
 impl DrawingCtx {
     pub fn new(
-        node: Option<&RsvgNode>,
+        node: Option<&Node>,
         cr: &cairo::Context,
         viewport: Rect,
         dpi: Dpi,
@@ -245,8 +242,7 @@ impl DrawingCtx {
         let top_rect = &view_box_stack[last].0;
 
         ViewParams {
-            dpi_x: self.dpi.x(),
-            dpi_y: self.dpi.y(),
+            dpi: self.dpi,
             view_box_width: top_rect.width(),
             view_box_height: top_rect.height(),
             view_box_stack: None,
@@ -266,14 +262,19 @@ impl DrawingCtx {
             .push(ViewBox(Rect::from_size(width, height)));
 
         ViewParams {
-            dpi_x: self.dpi.x(),
-            dpi_y: self.dpi.y(),
+            dpi: self.dpi,
             view_box_width: width,
             view_box_height: height,
             view_box_stack: Some(Rc::downgrade(&self.view_box_stack)),
         }
     }
 
+    /// Creates a new coordinate space inside a viewport.
+    ///
+    /// Note that this actually changes the `draw_ctx.cr`'s transformation to match
+    /// the new coordinate space, but the old one is not restored after the
+    /// result's `ViewParams` is dropped.  Thus, this function must be called
+    /// inside `draw_ctx.with_saved_cr` or `draw_ctx.with_discrete_layer`.
     pub fn push_new_viewport(
         &self,
         vbox: Option<ViewBox>,
@@ -330,7 +331,7 @@ impl DrawingCtx {
 
     fn clip_to_node(
         &mut self,
-        clip_node: &Option<RsvgNode>,
+        clip_node: &Option<Node>,
         acquired_nodes: &mut AcquiredNodes,
         bbox: &BoundingBox,
     ) -> Result<(), RenderingError> {
@@ -339,7 +340,7 @@ impl DrawingCtx {
         }
 
         let node = clip_node.as_ref().unwrap();
-        let units = node.borrow().get_impl::<ClipPath>().get_units();
+        let units = borrow_element_as!(node, ClipPath).get_units();
 
         if let Ok(transform) = bbox.rect_to_transform(units) {
             let cascaded = CascadedValues::new_from_node(node);
@@ -367,7 +368,7 @@ impl DrawingCtx {
     fn generate_cairo_mask(
         &mut self,
         mask: &Mask,
-        mask_node: &RsvgNode,
+        mask_node: &Node,
         transform: Transform,
         bbox: &BoundingBox,
         acquired_nodes: &mut AcquiredNodes,
@@ -396,7 +397,7 @@ impl DrawingCtx {
         };
 
         let mask_transform = mask_node
-            .borrow()
+            .borrow_element()
             .get_transform()
             .post_transform(&transform);
 
@@ -457,7 +458,7 @@ impl DrawingCtx {
             res?;
         }
 
-        let Opacity(opacity) = values.opacity;
+        let Opacity(opacity) = values.opacity();
 
         let mask = SharedImageSurface::wrap(mask_content_surface, SurfaceType::SRgb)?
             .to_mask(opacity)?
@@ -468,7 +469,7 @@ impl DrawingCtx {
 
     pub fn with_discrete_layer(
         &mut self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         values: &ComputedValues,
         clipping: bool,
@@ -481,17 +482,23 @@ impl DrawingCtx {
             draw_fn(acquired_nodes, self)
         } else {
             self.with_saved_cr(&mut |dc| {
-                let clip_uri = values.clip_path.0.get();
-                let mask = values.mask.0.get();
+                let clip_path_value = values.clip_path();
+                let mask_value = values.mask();
+
+                let clip_uri = clip_path_value.0.get();
+                let mask = mask_value.0.get();
 
                 // The `filter` property does not apply to masks.
-                let filter = if node.borrow().get_type() == NodeType::Mask {
-                    None
+                let filter = if node.is_element() {
+                    match *node.borrow_element() {
+                        Element::Mask(_) => None,
+                        _ => values.filter().0.get().cloned(),
+                    }
                 } else {
-                    values.filter.0.get()
+                    values.filter().0.get().cloned()
                 };
 
-                let UnitInterval(opacity) = values.opacity.0;
+                let UnitInterval(opacity) = values.opacity().0;
 
                 let affine_at_start = dc.get_transform();
 
@@ -552,7 +559,7 @@ impl DrawingCtx {
                         let img_surface = dc
                             .run_filter(
                                 acquired_nodes,
-                                filter_uri,
+                                &filter_uri,
                                 node,
                                 values,
                                 child_surface,
@@ -581,26 +588,37 @@ impl DrawingCtx {
                     // Mask
 
                     if let Some(fragment) = mask {
-                        if let Ok(acquired) = acquired_nodes.acquire(fragment, &[NodeType::Mask]) {
+                        if let Ok(acquired) = acquired_nodes.acquire(fragment) {
                             let mask_node = acquired.get();
 
-                            res = res.and_then(|bbox| {
-                                dc.generate_cairo_mask(
-                                    &mask_node.borrow().get_impl::<Mask>(),
-                                    &mask_node,
-                                    affines.for_temporary_surface,
-                                    &bbox,
-                                    acquired_nodes,
-                                )
-                                .and_then(|mask_surf| {
-                                    if let Some(surf) = mask_surf {
-                                        dc.cr.set_matrix(affines.compositing.into());
-                                        dc.cr.mask_surface(&surf, 0.0, 0.0);
-                                    }
-                                    Ok(())
-                                })
-                                .map(|_: ()| bbox)
-                            });
+                            match *mask_node.borrow_element() {
+                                Element::Mask(ref m) => {
+                                    res = res.and_then(|bbox| {
+                                        dc.generate_cairo_mask(
+                                            &m,
+                                            &mask_node,
+                                            affines.for_temporary_surface,
+                                            &bbox,
+                                            acquired_nodes,
+                                        )
+                                        .and_then(|mask_surf| {
+                                            if let Some(surf) = mask_surf {
+                                                dc.cr.set_matrix(affines.compositing.into());
+                                                dc.cr.mask_surface(&surf, 0.0, 0.0);
+                                            }
+                                            Ok(())
+                                        })
+                                        .map(|_: ()| bbox)
+                                    });
+                                }
+                                _ => {
+                                    rsvg_log!(
+                                        "element {} references \"{}\" which is not a mask",
+                                        node,
+                                        fragment
+                                    );
+                                }
+                            }
                         } else {
                             rsvg_log!(
                                 "element {} references nonexistent mask \"{}\"",
@@ -743,46 +761,50 @@ impl DrawingCtx {
         &mut self,
         acquired_nodes: &mut AcquiredNodes,
         filter_uri: &Fragment,
-        node: &RsvgNode,
+        node: &Node,
         values: &ComputedValues,
         child_surface: SharedImageSurface,
         node_bbox: BoundingBox,
     ) -> Result<SharedImageSurface, RenderingError> {
-        match acquired_nodes.acquire(filter_uri, &[NodeType::Filter]) {
+        match acquired_nodes.acquire(filter_uri) {
             Ok(acquired) => {
                 let filter_node = acquired.get();
 
-                if !filter_node.borrow().is_in_error() {
-                    // FIXME: deal with out of memory here
-                    filters::render(
-                        &filter_node,
-                        values,
-                        child_surface,
-                        acquired_nodes,
-                        self,
-                        node_bbox,
-                    )
-                } else {
-                    Ok(child_surface)
+                match *filter_node.borrow_element() {
+                    Element::Filter(_) => {
+                        return filters::render(
+                            &filter_node,
+                            values,
+                            child_surface,
+                            acquired_nodes,
+                            self,
+                            node_bbox,
+                        );
+                    }
+                    _ => {
+                        rsvg_log!(
+                            "element {} will not be rendered since \"{}\" is not a filter",
+                            node,
+                            filter_uri,
+                        );
+                    }
                 }
             }
-
-            Err(_) => {
+            _ => {
                 rsvg_log!(
                     "element {} will not be rendered since its filter \"{}\" was not found",
                     node,
                     filter_uri,
                 );
-
-                // Non-existing filters must act as null filters (that is, an
-                // empty surface is returned).
-                Ok(SharedImageSurface::empty(
-                    child_surface.width(),
-                    child_surface.height(),
-                    child_surface.surface_type(),
-                )?)
             }
         }
+
+        // Non-existing filters must act as null filters (an empty surface is returned).
+        Ok(SharedImageSurface::empty(
+            child_surface.width(),
+            child_surface.height(),
+            child_surface.surface_type(),
+        )?)
     }
 
     fn set_color(
@@ -820,42 +842,35 @@ impl DrawingCtx {
             } => {
                 let mut had_paint_server = false;
 
-                match acquire_paint_server(acquired_nodes, iri) {
+                match acquired_nodes.acquire(iri) {
                     Ok(acquired) => {
                         let node = acquired.get();
 
-                        had_paint_server = match node.borrow().get_type() {
-                            NodeType::LinearGradient => node
-                                .borrow()
-                                .get_impl::<LinearGradient>()
-                                .resolve_fallbacks_and_set_pattern(
-                                    &node,
-                                    acquired_nodes,
-                                    self,
-                                    opacity,
-                                    bbox,
-                                )?,
-                            NodeType::RadialGradient => node
-                                .borrow()
-                                .get_impl::<RadialGradient>()
-                                .resolve_fallbacks_and_set_pattern(
-                                    &node,
-                                    acquired_nodes,
-                                    self,
-                                    opacity,
-                                    bbox,
-                                )?,
-                            NodeType::Pattern => node
-                                .borrow()
-                                .get_impl::<Pattern>()
-                                .resolve_fallbacks_and_set_pattern(
-                                    &node,
-                                    acquired_nodes,
-                                    self,
-                                    opacity,
-                                    bbox,
-                                )?,
-                            _ => unreachable!(),
+                        assert!(node.is_element());
+
+                        had_paint_server = match *node.borrow_element() {
+                            Element::LinearGradient(ref g) => g.resolve_fallbacks_and_set_pattern(
+                                &node,
+                                acquired_nodes,
+                                self,
+                                opacity,
+                                bbox,
+                            )?,
+                            Element::RadialGradient(ref g) => g.resolve_fallbacks_and_set_pattern(
+                                &node,
+                                acquired_nodes,
+                                self,
+                                opacity,
+                                bbox,
+                            )?,
+                            Element::Pattern(ref p) => p.resolve_fallbacks_and_set_pattern(
+                                &node,
+                                acquired_nodes,
+                                self,
+                                opacity,
+                                bbox,
+                            )?,
+                            _ => false,
                         }
                     }
 
@@ -891,12 +906,12 @@ impl DrawingCtx {
     pub fn setup_cr_for_stroke(&self, cr: &cairo::Context, values: &ComputedValues) {
         let params = self.get_view_params();
 
-        cr.set_line_width(values.stroke_width.0.normalize(values, &params));
-        cr.set_miter_limit(values.stroke_miterlimit.0);
-        cr.set_line_cap(cairo::LineCap::from(values.stroke_line_cap));
-        cr.set_line_join(cairo::LineJoin::from(values.stroke_line_join));
+        cr.set_line_width(values.stroke_width().0.normalize(values, &params));
+        cr.set_miter_limit(values.stroke_miterlimit().0);
+        cr.set_line_cap(cairo::LineCap::from(values.stroke_line_cap()));
+        cr.set_line_join(cairo::LineJoin::from(values.stroke_line_join()));
 
-        if let StrokeDasharray(Dasharray::Array(ref dashes)) = values.stroke_dasharray {
+        if let StrokeDasharray(Dasharray::Array(ref dashes)) = values.stroke_dasharray() {
             let normalized_dashes: Vec<f64> = dashes
                 .iter()
                 .map(|l| l.normalize(values, &params))
@@ -905,7 +920,7 @@ impl DrawingCtx {
             let total_length = normalized_dashes.iter().fold(0.0, |acc, &len| acc + len);
 
             if total_length > 0.0 {
-                let offset = values.stroke_dashoffset.0.normalize(values, &params);
+                let offset = values.stroke_dashoffset().0.normalize(values, &params);
                 cr.set_dash(&normalized_dashes, offset);
             } else {
                 cr.set_dash(&[], 0.0);
@@ -919,7 +934,7 @@ impl DrawingCtx {
         acquired_nodes: &mut AcquiredNodes,
         values: &ComputedValues,
     ) -> Result<BoundingBox, RenderingError> {
-        cr.set_antialias(cairo::Antialias::from(values.shape_rendering));
+        cr.set_antialias(cairo::Antialias::from(values.shape_rendering()));
 
         self.setup_cr_for_stroke(cr, values);
 
@@ -929,19 +944,19 @@ impl DrawingCtx {
         // coordinate system in patterns.
         let bbox = compute_stroke_and_fill_box(cr, values);
 
-        let current_color = values.color.0;
+        let current_color = values.color().0;
 
         let res = self
             .set_source_paint_server(
                 acquired_nodes,
-                &values.fill.0,
-                values.fill_opacity.0,
+                &values.fill().0,
+                values.fill_opacity().0,
                 &bbox,
                 current_color,
             )
             .and_then(|had_paint_server| {
                 if had_paint_server {
-                    if values.stroke.0 == PaintServer::None {
+                    if values.stroke().0 == PaintServer::None {
                         cr.fill();
                     } else {
                         cr.fill_preserve();
@@ -953,8 +968,8 @@ impl DrawingCtx {
             .and_then(|_| {
                 self.set_source_paint_server(
                     acquired_nodes,
-                    &values.stroke.0,
-                    values.stroke_opacity.0,
+                    &values.stroke().0,
+                    values.stroke_opacity().0,
                     &bbox,
                     current_color,
                 )
@@ -975,37 +990,31 @@ impl DrawingCtx {
 
     pub fn draw_path(
         &mut self,
-        builder: &PathBuilder,
-        node: &RsvgNode,
+        path: &Path,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         values: &ComputedValues,
         markers: Markers,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
-        if !builder.is_empty() {
+        if !path.is_empty() {
             let bbox =
                 self.with_discrete_layer(node, acquired_nodes, values, clipping, &mut |an, dc| {
                     let cr = dc.get_cairo_context();
 
-                    builder.to_cairo(&cr)?;
+                    path.to_cairo(&cr)?;
 
                     if clipping {
-                        cr.set_fill_rule(cairo::FillRule::from(values.clip_rule));
+                        cr.set_fill_rule(cairo::FillRule::from(values.clip_rule()));
                         Ok(dc.empty_bbox())
                     } else {
-                        cr.set_fill_rule(cairo::FillRule::from(values.fill_rule));
+                        cr.set_fill_rule(cairo::FillRule::from(values.fill_rule()));
                         dc.stroke_and_fill(&cr, an, values)
                     }
                 })?;
 
             if markers == Markers::Yes {
-                marker::render_markers_for_path_builder(
-                    builder,
-                    self,
-                    acquired_nodes,
-                    values,
-                    clipping,
-                )?;
+                marker::render_markers_for_path(path, self, acquired_nodes, values, clipping)?;
             }
 
             Ok(bbox)
@@ -1057,7 +1066,7 @@ impl DrawingCtx {
 
     pub fn draw_node_to_surface(
         &mut self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         affine: Transform,
@@ -1088,7 +1097,7 @@ impl DrawingCtx {
 
     pub fn draw_node_from_stack(
         &mut self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         clipping: bool,
@@ -1117,14 +1126,12 @@ impl DrawingCtx {
 
     pub fn draw_from_use_node(
         &mut self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
+        link: Option<&Fragment>,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
-        let node_data = node.borrow();
-        let use_ = node_data.get_impl::<Use>();
-
         // <use> is an element that is used directly, unlike
         // <pattern>, which is used through a fill="url(#...)"
         // reference.  However, <use> will always reference another
@@ -1141,12 +1148,11 @@ impl DrawingCtx {
             }
         })?;
 
-        let link = use_.get_link();
         if link.is_none() {
             return Ok(self.empty_bbox());
         }
 
-        let acquired = match acquired_nodes.acquire(link.unwrap(), &[]) {
+        let acquired = match acquired_nodes.acquire(link.unwrap()) {
             Ok(acquired) => acquired,
 
             Err(AcquireError::CircularReference(node)) => {
@@ -1168,7 +1174,7 @@ impl DrawingCtx {
 
         let values = cascaded.get();
         let params = self.get_view_params();
-        let use_rect = use_.get_rect(values, &params);
+        let use_rect = borrow_element_as!(node, Use).get_rect(values, &params);
 
         // width or height set to 0 disables rendering of the element
         // https://www.w3.org/TR/SVG/struct.html#UseElementWidthAttribute
@@ -1178,46 +1184,56 @@ impl DrawingCtx {
 
         let child = acquired.get();
 
-        if child.borrow().get_type() != NodeType::Symbol {
-            let cr = self.get_cairo_context();
-            cr.translate(use_rect.x0, use_rect.y0);
+        // if it is a symbol
+        if child.is_element() {
+            let elt = child.borrow_element();
 
-            self.with_discrete_layer(node, acquired_nodes, values, clipping, &mut |an, dc| {
-                dc.draw_node_from_stack(
-                    &child,
-                    an,
-                    &CascadedValues::new_from_values(&child, values),
+            if let Element::Symbol(ref symbol) = *elt {
+                let clip_mode = if !values.is_overflow()
+                    || (values.overflow() == Overflow::Visible
+                        && elt.get_specified_values().is_overflow())
+                {
+                    Some(ClipMode::ClipToVbox)
+                } else {
+                    None
+                };
+
+                return self.with_discrete_layer(
+                    node,
+                    acquired_nodes,
+                    values,
                     clipping,
-                )
-            })
-        } else {
-            let node_data = child.borrow();
-            let symbol = node_data.get_impl::<Symbol>();
+                    &mut |an, dc| {
+                        let _params = dc.push_new_viewport(
+                            symbol.get_viewbox(),
+                            use_rect,
+                            symbol.get_preserve_aspect_ratio(),
+                            clip_mode,
+                        );
 
-            let clip_mode = if !values.is_overflow()
-                || (values.overflow == Overflow::Visible && child.borrow().is_overflow())
-            {
-                Some(ClipMode::ClipToVbox)
-            } else {
-                None
-            };
-
-            self.with_discrete_layer(node, acquired_nodes, values, clipping, &mut |an, dc| {
-                let _params = dc.push_new_viewport(
-                    symbol.get_viewbox(),
-                    use_rect,
-                    symbol.get_preserve_aspect_ratio(),
-                    clip_mode,
+                        child.draw_children(
+                            an,
+                            &CascadedValues::new_from_values(&child, values),
+                            dc,
+                            clipping,
+                        )
+                    },
                 );
+            }
+        };
 
-                child.draw_children(
-                    an,
-                    &CascadedValues::new_from_values(&child, values),
-                    dc,
-                    clipping,
-                )
-            })
-        }
+        // all other nodes
+        let cr = self.get_cairo_context();
+        cr.translate(use_rect.x0, use_rect.y0);
+
+        self.with_discrete_layer(node, acquired_nodes, values, clipping, &mut |an, dc| {
+            dc.draw_node_from_stack(
+                &child,
+                an,
+                &CascadedValues::new_from_values(&child, values),
+                clipping,
+            )
+        })
     }
 }
 
@@ -1270,17 +1286,22 @@ impl CompositingAffines {
     }
 }
 
-// Returns (clip_in_user_space, clip_in_object_space), both Option<RsvgNode>
+// Returns (clip_in_user_space, clip_in_object_space), both Option<Node>
 fn get_clip_in_user_and_object_space(
     acquired_nodes: &mut AcquiredNodes,
     clip_uri: Option<&Fragment>,
-) -> (Option<RsvgNode>, Option<RsvgNode>) {
+) -> (Option<Node>, Option<Node>) {
     clip_uri
-        .and_then(|fragment| acquired_nodes.acquire(fragment, &[NodeType::ClipPath]).ok())
+        .and_then(|fragment| {
+            acquired_nodes
+                .acquire(fragment)
+                .ok()
+                .filter(|a| is_element_of_type!(*a.get(), ClipPath))
+        })
         .and_then(|acquired| {
             let clip_node = acquired.get().clone();
 
-            let units = clip_node.borrow().get_impl::<ClipPath>().get_units();
+            let units = borrow_element_as!(clip_node, ClipPath).get_units();
 
             match units {
                 CoordUnits::UserSpaceOnUse => Some((Some(clip_node), None)),
@@ -1290,20 +1311,6 @@ fn get_clip_in_user_and_object_space(
         .unwrap_or((None, None))
 }
 
-fn acquire_paint_server(
-    acquired_nodes: &mut AcquiredNodes,
-    fragment: &Fragment,
-) -> Result<AcquiredNode, AcquireError> {
-    acquired_nodes.acquire(
-        fragment,
-        &[
-            NodeType::LinearGradient,
-            NodeType::RadialGradient,
-            NodeType::Pattern,
-        ],
-    )
-}
-
 fn compute_stroke_and_fill_box(cr: &cairo::Context, values: &ComputedValues) -> BoundingBox {
     let affine = Transform::from(cr.get_matrix());
 
@@ -1331,7 +1338,7 @@ fn compute_stroke_and_fill_box(cr: &cairo::Context, values: &ComputedValues) ->
 
     // Bounding box for stroke
 
-    if values.stroke.0 != PaintServer::None {
+    if values.stroke().0 != PaintServer::None {
         let (x0, y0, x1, y1) = cr.stroke_extents();
         let sb = BoundingBox::new()
             .with_transform(affine)
