@@ -20,10 +20,9 @@ use crate::document::{Document, DocumentBuilder};
 use crate::error::LoadingError;
 use crate::io::{self, get_input_stream_for_loading};
 use crate::limits::MAX_LOADED_ELEMENTS;
-use crate::node::{NodeType, RsvgNode};
+use crate::node::{Node, NodeBorrow};
 use crate::property_bag::PropertyBag;
-use crate::style::{Style, StyleType};
-use crate::text::NodeChars;
+use crate::style::StyleType;
 use crate::xml2_load::Xml2Parser;
 
 #[derive(Clone)]
@@ -101,7 +100,7 @@ struct XmlStateInner {
     document_builder: Option<DocumentBuilder>,
     num_loaded_elements: usize,
     context_stack: Vec<Context>,
-    current_node: Option<RsvgNode>,
+    current_node: Option<Node>,
 
     entities: HashMap<String, XmlEntityPtr>,
 }
@@ -362,11 +361,7 @@ impl XmlState {
         let mut inner = self.inner.borrow_mut();
         let current_node = inner.current_node.as_ref().unwrap();
 
-        assert!(current_node.borrow().get_type() == NodeType::Style);
-
-        let style_type = current_node
-            .borrow()
-            .get_impl::<Style>()
+        let style_type = borrow_element_as!(current_node, Style)
             .style_type()
             .unwrap_or(StyleType::TextCss);
 
@@ -374,10 +369,10 @@ impl XmlState {
             let stylesheet_text = current_node
                 .children()
                 .map(|child| {
-                    let child_borrow = child.borrow();
-
-                    assert!(child_borrow.get_type() == NodeType::Chars);
-                    child_borrow.get_impl::<NodeChars>().get_string()
+                    // Note that here we assume that the only children of <style>
+                    // are indeed text nodes.
+                    let child_borrow = child.borrow_chars();
+                    child_borrow.get_string()
                 })
                 .collect::<String>();
 
@@ -680,9 +675,9 @@ fn parse_xml_stylesheet_processing_instruction(data: &str) -> Result<Vec<(String
     let pi_data = pi_data.borrow();
 
     if pi_data.error {
-        return Err(());
+        Err(())
     } else {
-        return Ok(pi_data.attributes.clone());
+        Ok(pi_data.attributes.clone())
     }
 }
 
