@@ -238,15 +238,10 @@ test_utils_get_test_data_path (void)
 static int
 compare_files (gconstpointer a, gconstpointer b)
 {
-    GFile *file1 = G_FILE (a);
-    GFile *file2 = G_FILE (b);
-    char *uri1, *uri2;
-    int result;
+    char *uri1 = g_file_get_uri (G_FILE(a));
+    char *uri2 = g_file_get_uri (G_FILE(b));
 
-    uri1 = g_file_get_uri (file1);
-    uri2 = g_file_get_uri (file2);
-
-    result = strcmp (uri1, uri2);
+    int result = strcmp (uri1, uri2);
 
     g_free (uri1);
     g_free (uri2);
@@ -263,10 +258,9 @@ test_utils_add_test_for_all_files (const gchar   *prefix,
 {
     GFileEnumerator *enumerator;
     GFileInfo *info;
-    GList *l, *files;
+    GList *l, *files = NULL;
     GError *error = NULL;
 
-
     if (g_file_query_file_type (file, 0, NULL) != G_FILE_TYPE_DIRECTORY)
     {
         gchar *test_path;
@@ -283,13 +277,12 @@ test_utils_add_test_for_all_files (const gchar   *prefix,
         g_test_add_data_func_full (test_path, g_object_ref (file), test_func, g_object_unref);
 
         g_free (test_path);
+
         return;
     }
 
-
     enumerator = g_file_enumerate_children (file, G_FILE_ATTRIBUTE_STANDARD_NAME, 0, NULL, &error);
     g_assert_no_error (error);
-    files = NULL;
 
     while ((info = g_file_enumerator_next_file (enumerator, NULL, &error)))
     {
@@ -323,6 +316,8 @@ create_font_config_for_testing (void)
 {
     const char *font_paths[] =
     {
+        "NotoSerifItalic-2Y8w.ttf",
+        "NotoSerif-rpn9.ttf",
         "Roboto-Regular.ttf",
         "Roboto-Italic.ttf",
         "Roboto-Bold.ttf",
